/* archive-fixup.c
 *
 ****************************************************************
 * Copyright (C) 2003 Tom Lord
 *
 * See the file "COPYING" for further information about
 * the copyright and warranty status of this work.
 */


#include "config-options.h"
#include "po/gettext.h"
#include "hackerlab/cmd/main.h"
#include "libarch/namespace.h"
#include "libarch/project-tree.h"
#include "libarch/my.h"
#include "libarch/archive.h"
#include "commands/cmd.h"
#include "commands/archive-fixup.h"
#include "commands/version.h"



static t_uchar * usage = N_("[options] [archive]");

#define OPTS(OP) \
  OP (opt_help_msg, "h", "help", 0, \
      N_("Display a help message and exit.")) \
  OP (opt_long_help, "H", 0, 0, \
      N_("Display a verbose help message and exit.")) \
  OP (opt_version, "V", "version", 0, \
      N_("Display a release identifier string\n" \
      "and exit."))

t_uchar arch_cmd_archive_fixup_help[] = N_("fix ancillary files (e.g. .listing files) in an archive\n"
                                         "This is a kind of \"catch-all\" command to fix the non-standard,\n"
                                         "non-transactional state of an archive.  Currently its sole\n"
                                         "function is to repair .listing files in archives that use them.\n");

enum options
{
  OPTS (OPT_ENUM)
};

static struct opt_desc opts[] =
{
  OPTS (OPT_DESC)
    {-1, 0, 0, 0, 0}
};



int
arch_cmd_archive_fixup (t_uchar *program_name, int argc, char * argv[])
{
  int o;
  struct opt_parsed * option;
  int result = 0;

  safe_buffer_fd (1, 0, O_WRONLY, 0);

  option = 0;

  while (1)
    {
      o = opt_standard (lim_use_must_malloc, &option, opts, &argc, argv, program_name, usage, libarch_version_string, arch_cmd_archive_fixup_help, opt_help_msg, opt_long_help, opt_version);
      if (o == opt_none)
        break;
      switch (o)
        {
        default:
          safe_printfmt (2, "unhandled option `%s'\n", option->opt_string);
          panic ("internal error parsing arguments");

        usage_error:
          opt_usage (2, argv[0], program_name, usage, 1);
          exit (1);

          /* bogus_arg: */
          safe_printfmt (2, "ill-formed argument for `%s' (`%s')\n", option->opt_string, option->arg_string);
          goto usage_error;

        }
    }

  if (argc < 1 || argc > 2)
    goto usage_error;

  {
    struct arch_archive * arch = 0;
    t_uchar * archive;
    if (argc == 1)
	archive = arch_my_default_archive (NULL);
    else
	archive = str_save (0, argv[1]);

    arch = arch_archive_connect_writeable_branch (archive, NULL);
    if (arch)
	arch_archive_repair_non_txnal (1, arch);
    else
      {
	safe_printfmt (2, _("no writable mirrors available for %s\n"), archive);
	result = 2;
      }

    arch_archive_close (arch);
    lim_free (0, archive);
  }

  return result;
}




/* tag: Tom Lord Fri Jun 13 23:48:45 2003 (cmd-archive-fixup.c)
 */
